@@ -2,7 +2,8 @@ package Protocol::HTTP2::Stream;
 use strict;
 use warnings;
 use Hash::MultiValue;
-use Protocol::HTTP2::Constants qw(:states :endpoints);
+use Protocol::HTTP2::Constants qw(:states :endpoints :settings :frame_types
+  :limits);
 use Protocol::HTTP2::HeaderCompression qw( headers_decode );
 use Protocol::HTTP2::Trace qw(tracer);
 
@@ -14,7 +15,11 @@ sub new_stream {
 
     $self->{last_stream} += 2
       if exists $self->{streams}->{ $self->{type} == CLIENT ? 1 : 2 };
-    $self->{streams}->{ $self->{last_stream} } = { 'state' => IDLE };
+    $self->{streams}->{ $self->{last_stream} } = {
+        'state'    => IDLE,
+        'fcw_recv' => $self->setting(SETTINGS_INITIAL_WINDOW_SIZE),
+        'fcw_send' => $self->setting(SETTINGS_INITIAL_WINDOW_SIZE),
+    };
     return $self->{last_stream};
 }
 
@@ -28,7 +33,11 @@ sub new_peer_stream {
         return undef;
     }
     $self->{last_peer_stream} = $stream_id;
-    $self->{streams}->{$stream_id} = { 'state' => IDLE };
+    $self->{streams}->{$stream_id} = {
+        'state'    => IDLE,
+        'fcw_recv' => $self->setting(SETTINGS_INITIAL_WINDOW_SIZE),
+        'fcw_send' => $self->setting(SETTINGS_INITIAL_WINDOW_SIZE),
+    };
     $self->{on_new_peer_stream}->($stream_id)
       if exists $self->{on_new_peer_stream};
 
@@ -182,4 +191,76 @@ sub stream_headers_done {
     return 1;
 }
 
+# RST_STREAM for stream errors
+sub stream_error {
+    my ( $self, $stream_id, $error ) = @_;
+    $self->enqueue(
+        $self->frame_encode( RST_STREAM, 0, $stream_id, $error, '' ) );
+}
+
+# Flow control windown of stream
+sub stream_fcw_send {
+    shift->_stream_fcw( 'send', @_ );
+}
+
+sub stream_fcw_recv {
+    shift->_stream_fcw( 'recv', @_ );
+}
+
+sub _stream_fcw {
+    my $self      = shift;
+    my $dir       = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    my $s = $self->{streams}->{$stream_id};
+
+    if (@_) {
+        $s->{ 'fcw_' . $dir } += shift;
+        tracer->debug( "Stream $stream_id fcw_$dir now is "
+              . $self->{ 'fcw_' . $dir }
+              . "\n" );
+    }
+    $s->{ 'fcw_' . $dir };
+}
+
+sub stream_fcw_update {
+    my ( $self, $stream_id ) = @_;
+
+    # TODO: check size of data of stream  in memory
+    tracer->debug("update fcw recv of stream $stream_id\n");
+    $self->stream_fcw_recv( $stream_id, DEFAULT_INITIAL_WINDOW_SIZE );
+    $self->enqueue(
+        $self->frame_encode( WINDOW_UPDATE, 0, $stream_id,
+            DEFAULT_INITIAL_WINDOW_SIZE
+        )
+    );
+}
+
+sub stream_blocked_data {
+    my $self      = shift;
+    my $stream_id = shift;
+    return undef unless exists $self->{streams}->{$stream_id};
+    my $s = $self->{streams}->{$stream_id};
+
+    if (@_) {
+        $s->{blocked_data} .= shift;
+        $self->enqueue( $self->frame_encode( BLOCKED, 0, $stream_id, '' ) )
+          if $self->stream_fcw_send($stream_id) == 0;
+        $self->enqueue( $self->frame_encode( BLOCKED, 0, 0, '' ) )
+          if $self->fcw_send == 0;
+    }
+    $s->{blocked_data};
+}
+
+sub stream_send_blocked {
+    my ( $self, $stream_id ) = @_;
+    my $blocked_data = $self->stream_blocked_data($stream_id);
+    return undef
+      unless defined $blocked_data
+      && length($blocked_data)
+      && $self->stream_fcw_send($stream_id) != 0;
+
+    $self->send_data( $stream_id, $blocked_data );
+}
+
 1;
